@@ -19,6 +19,7 @@ License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
 @bug Need to add unit tests
+@todo Look into an actor system
 @bug Session timeouts are bad, possible solutions:
 	@li For every X minutes, if logged in, attempt login
 	@li Restructure so code handling login/dial/sms is beneath everything else and login attempts are made if those fail
@@ -127,9 +128,13 @@ class Dialcentral(object):
 			self._window = hildon.Window()
 			oldWindow.get_child().reparent(self._window)
 			self._app.add_window(self._window)
-			self._widgetTree.get_widget("usernameentry").set_property('hildon-input-mode', 7)
-			self._widgetTree.get_widget("passwordentry").set_property('hildon-input-mode', 7|(1 << 29))
-			self._widgetTree.get_widget("callbackcombo").get_child().set_property('hildon-input-mode', (1 << 4))
+
+			try:
+				self._widgetTree.get_widget("usernameentry").set_property('hildon-input-mode', 7)
+				self._widgetTree.get_widget("passwordentry").set_property('hildon-input-mode', 7|(1 << 29))
+				self._widgetTree.get_widget("callbackcombo").get_child().set_property('hildon-input-mode', (1 << 4))
+			except TypeError, e:
+				warnings.warn(e.message)
 			hildon.hildon_helper_set_thumb_scrollbar(self._widgetTree.get_widget('recent_scrolledwindow'), True)
 			hildon.hildon_helper_set_thumb_scrollbar(self._widgetTree.get_widget('message_scrolledwindow'), True)
 			hildon.hildon_helper_set_thumb_scrollbar(self._widgetTree.get_widget('contacts_scrolledwindow'), True)
@@ -156,10 +161,9 @@ class Dialcentral(object):
 		}
 		self._widgetTree.signal_autoconnect(callbackMapping)
 
-		if self._window:
-			self._window.connect("destroy", self._on_close)
-			self._window.show_all()
-			self._window.set_default_size(800, 300)
+		self._window.connect("destroy", self._on_close)
+		self._window.set_default_size(800, 300)
+		self._window.show_all()
 
 		backgroundSetup = threading.Thread(target=self._idle_setup)
 		backgroundSetup.setDaemon(True)
@@ -169,147 +173,155 @@ class Dialcentral(object):
 		"""
 		If something can be done after the UI loads, push it here so it's not blocking the UI
 		"""
-		# Barebones UI handlers
-		import null_backend
-		import null_views
-
-		self._phoneBackends = {self.NULL_BACKEND: null_backend.NullDialer()}
-		with gtk_toolbox.gtk_lock():
-			self._dialpads = {self.NULL_BACKEND: null_views.Dialpad(self._widgetTree)}
-			self._accountViews = {self.NULL_BACKEND: null_views.AccountInfo(self._widgetTree)}
-			self._recentViews = {self.NULL_BACKEND: null_views.RecentCallsView(self._widgetTree)}
-			self._messagesViews = {self.NULL_BACKEND: null_views.MessagesView(self._widgetTree)}
-			self._contactsViews = {self.NULL_BACKEND: null_views.ContactsView(self._widgetTree)}
-
-			self._dialpads[self._selectedBackendId].enable()
-			self._accountViews[self._selectedBackendId].enable()
-			self._recentViews[self._selectedBackendId].enable()
-			self._messagesViews[self._selectedBackendId].enable()
-			self._contactsViews[self._selectedBackendId].enable()
-
-		# Setup maemo specifics
 		try:
-			import osso
-		except ImportError:
-			osso = None
-		self._osso = None
-		if osso is not None:
-			self._osso = osso.Context(constants.__app_name__, constants.__version__, False)
-			device = osso.DeviceState(self._osso)
-			device.set_device_state_callback(self._on_device_state_change, 0)
-		else:
-			pass # warnings.warn("No OSSO", UserWarning)
+			# Barebones UI handlers
+			import null_backend
+			import null_views
 
-		# Setup maemo specifics
-		try:
-			import conic
-		except ImportError:
-			conic = None
-		self._connection = None
-		if conic is not None:
-			self._connection = conic.Connection()
-			self._connection.connect("connection-event", self._on_connection_change, constants.__app_magic__)
-			self._connection.request_connection(conic.CONNECT_FLAG_NONE)
-		else:
-			pass # warnings.warn("No Internet Connectivity API ", UserWarning)
+			self._phoneBackends = {self.NULL_BACKEND: null_backend.NullDialer()}
+			with gtk_toolbox.gtk_lock():
+				self._dialpads = {self.NULL_BACKEND: null_views.Dialpad(self._widgetTree)}
+				self._accountViews = {self.NULL_BACKEND: null_views.AccountInfo(self._widgetTree)}
+				self._recentViews = {self.NULL_BACKEND: null_views.RecentCallsView(self._widgetTree)}
+				self._messagesViews = {self.NULL_BACKEND: null_views.MessagesView(self._widgetTree)}
+				self._contactsViews = {self.NULL_BACKEND: null_views.ContactsView(self._widgetTree)}
 
-		# Setup costly backends
-		import gv_backend
-		import gc_backend
-		import file_backend
-		import evo_backend
-		import gc_views
+				self._dialpads[self._selectedBackendId].enable()
+				self._accountViews[self._selectedBackendId].enable()
+				self._recentViews[self._selectedBackendId].enable()
+				self._messagesViews[self._selectedBackendId].enable()
+				self._contactsViews[self._selectedBackendId].enable()
 
-		try:
-			os.makedirs(self._data_path)
-		except OSError, e:
-			if e.errno != 17:
+			# Setup maemo specifics
+			try:
+				import osso
+			except ImportError:
+				osso = None
+			self._osso = None
+			if osso is not None:
+				self._osso = osso.Context(constants.__app_name__, constants.__version__, False)
+				device = osso.DeviceState(self._osso)
+				device.set_device_state_callback(self._on_device_state_change, 0)
+			else:
+				pass # warnings.warn("No OSSO", UserWarning, 2)
+
+			# Setup maemo specifics
+			try:
+				import conic
+			except ImportError:
+				conic = None
+			self._connection = None
+			if conic is not None:
+				self._connection = conic.Connection()
+				self._connection.connect("connection-event", self._on_connection_change, constants.__app_magic__)
+				self._connection.request_connection(conic.CONNECT_FLAG_NONE)
+			else:
+				pass # warnings.warn("No Internet Connectivity API ", UserWarning)
+
+			# Setup costly backends
+			import gv_backend
+			import gc_backend
+			import file_backend
+			import evo_backend
+			import gc_views
+
+			try:
+				os.makedirs(self._data_path)
+			except OSError, e:
+				if e.errno != 17:
+					raise
+			gcCookiePath = os.path.join(self._data_path, "gc_cookies.txt")
+			gvCookiePath = os.path.join(self._data_path, "gv_cookies.txt")
+			self._defaultBackendId = self._guess_preferred_backend((
+				(self.GC_BACKEND, gcCookiePath),
+				(self.GV_BACKEND, gvCookiePath),
+			))
+
+			self._phoneBackends.update({
+				self.GC_BACKEND: gc_backend.GCDialer(gcCookiePath),
+				self.GV_BACKEND: gv_backend.GVDialer(gvCookiePath),
+			})
+			with gtk_toolbox.gtk_lock():
+				unifiedDialpad = gc_views.Dialpad(self._widgetTree, self._errorDisplay)
+				unifiedDialpad.set_number("")
+				self._dialpads.update({
+					self.GC_BACKEND: unifiedDialpad,
+					self.GV_BACKEND: unifiedDialpad,
+				})
+				self._accountViews.update({
+					self.GC_BACKEND: gc_views.AccountInfo(
+						self._widgetTree, self._phoneBackends[self.GC_BACKEND], self._errorDisplay
+					),
+					self.GV_BACKEND: gc_views.AccountInfo(
+						self._widgetTree, self._phoneBackends[self.GV_BACKEND], self._errorDisplay
+					),
+				})
+				self._recentViews.update({
+					self.GC_BACKEND: gc_views.RecentCallsView(
+						self._widgetTree, self._phoneBackends[self.GC_BACKEND], self._errorDisplay
+					),
+					self.GV_BACKEND: gc_views.RecentCallsView(
+						self._widgetTree, self._phoneBackends[self.GV_BACKEND], self._errorDisplay
+					),
+				})
+				self._messagesViews.update({
+					self.GC_BACKEND: null_views.MessagesView(self._widgetTree),
+					self.GV_BACKEND: gc_views.MessagesView(
+						self._widgetTree, self._phoneBackends[self.GV_BACKEND], self._errorDisplay
+					),
+				})
+				self._contactsViews.update({
+					self.GC_BACKEND: gc_views.ContactsView(
+						self._widgetTree, self._phoneBackends[self.GC_BACKEND], self._errorDisplay
+					),
+					self.GV_BACKEND: gc_views.ContactsView(
+						self._widgetTree, self._phoneBackends[self.GV_BACKEND], self._errorDisplay
+					),
+				})
+
+			evoBackend = evo_backend.EvolutionAddressBook()
+			fsContactsPath = os.path.join(self._data_path, "contacts")
+			fileBackend = file_backend.FilesystemAddressBookFactory(fsContactsPath)
+			for backendId in (self.GV_BACKEND, self.GC_BACKEND):
+				self._dialpads[backendId].number_selected = self._select_action
+				self._recentViews[backendId].number_selected = self._select_action
+				self._messagesViews[backendId].number_selected = self._select_action
+				self._contactsViews[backendId].number_selected = self._select_action
+
+				addressBooks = [
+					self._phoneBackends[backendId],
+					evoBackend,
+					fileBackend,
+				]
+				mergedBook = gc_views.MergedAddressBook(addressBooks, gc_views.MergedAddressBook.advanced_lastname_sorter)
+				self._contactsViews[backendId].append(mergedBook)
+				self._contactsViews[backendId].extend(addressBooks)
+				self._contactsViews[backendId].open_addressbook(*self._contactsViews[backendId].get_addressbooks().next()[0][0:2])
+
+			callbackMapping = {
+				"on_paste": self._on_paste,
+				"on_refresh": self._on_refresh,
+				"on_clearcookies_clicked": self._on_clearcookies_clicked,
+				"on_notebook_switch_page": self._on_notebook_switch_page,
+				"on_about_activate": self._on_about_activate,
+			}
+			self._widgetTree.signal_autoconnect(callbackMapping)
+
+			self._initDone = True
+
+			config = ConfigParser.SafeConfigParser()
+			config.read(self._user_settings)
+			with gtk_toolbox.gtk_lock():
+				self.load_settings(config)
+
+			gtk_toolbox.asynchronous_gtk_message(self._spawn_attempt_login)(2)
+		except StandardError, e:
+			warnings.warn(e.message, UserWarning, 2)
+		except BaseException, e:
+			try:
+				warnings.warn(e.message, UserWarning, 2)
+			finally:
 				raise
-		gcCookiePath = os.path.join(self._data_path, "gc_cookies.txt")
-		gvCookiePath = os.path.join(self._data_path, "gv_cookies.txt")
-		self._defaultBackendId = self._guess_preferred_backend((
-			(self.GC_BACKEND, gcCookiePath),
-			(self.GV_BACKEND, gvCookiePath),
-		))
-
-		self._phoneBackends.update({
-			self.GC_BACKEND: gc_backend.GCDialer(gcCookiePath),
-			self.GV_BACKEND: gv_backend.GVDialer(gvCookiePath),
-		})
-		with gtk_toolbox.gtk_lock():
-			unifiedDialpad = gc_views.Dialpad(self._widgetTree, self._errorDisplay)
-			unifiedDialpad.set_number("")
-			self._dialpads.update({
-				self.GC_BACKEND: unifiedDialpad,
-				self.GV_BACKEND: unifiedDialpad,
-			})
-			self._accountViews.update({
-				self.GC_BACKEND: gc_views.AccountInfo(
-					self._widgetTree, self._phoneBackends[self.GC_BACKEND], self._errorDisplay
-				),
-				self.GV_BACKEND: gc_views.AccountInfo(
-					self._widgetTree, self._phoneBackends[self.GV_BACKEND], self._errorDisplay
-				),
-			})
-			self._recentViews.update({
-				self.GC_BACKEND: gc_views.RecentCallsView(
-					self._widgetTree, self._phoneBackends[self.GC_BACKEND], self._errorDisplay
-				),
-				self.GV_BACKEND: gc_views.RecentCallsView(
-					self._widgetTree, self._phoneBackends[self.GV_BACKEND], self._errorDisplay
-				),
-			})
-			self._messagesViews.update({
-				self.GC_BACKEND: null_views.MessagesView(self._widgetTree),
-				self.GV_BACKEND: gc_views.MessagesView(
-					self._widgetTree, self._phoneBackends[self.GV_BACKEND], self._errorDisplay
-				),
-			})
-			self._contactsViews.update({
-				self.GC_BACKEND: gc_views.ContactsView(
-					self._widgetTree, self._phoneBackends[self.GC_BACKEND], self._errorDisplay
-				),
-				self.GV_BACKEND: gc_views.ContactsView(
-					self._widgetTree, self._phoneBackends[self.GV_BACKEND], self._errorDisplay
-				),
-			})
-
-		evoBackend = evo_backend.EvolutionAddressBook()
-		fsContactsPath = os.path.join(self._data_path, "contacts")
-		fileBackend = file_backend.FilesystemAddressBookFactory(fsContactsPath)
-		for backendId in (self.GV_BACKEND, self.GC_BACKEND):
-			self._dialpads[backendId].number_selected = self._select_action
-			self._recentViews[backendId].number_selected = self._select_action
-			self._messagesViews[backendId].number_selected = self._select_action
-			self._contactsViews[backendId].number_selected = self._select_action
-
-			addressBooks = [
-				self._phoneBackends[backendId],
-				evoBackend,
-				fileBackend,
-			]
-			mergedBook = gc_views.MergedAddressBook(addressBooks, gc_views.MergedAddressBook.advanced_lastname_sorter)
-			self._contactsViews[backendId].append(mergedBook)
-			self._contactsViews[backendId].extend(addressBooks)
-			self._contactsViews[backendId].open_addressbook(*self._contactsViews[backendId].get_addressbooks().next()[0][0:2])
-
-		callbackMapping = {
-			"on_paste": self._on_paste,
-			"on_refresh": self._on_refresh,
-			"on_clearcookies_clicked": self._on_clearcookies_clicked,
-			"on_notebook_switch_page": self._on_notebook_switch_page,
-			"on_about_activate": self._on_about_activate,
-		}
-		self._widgetTree.signal_autoconnect(callbackMapping)
-
-		self._initDone = True
-
-		config = ConfigParser.SafeConfigParser()
-		config.read(self._user_settings)
-		with gtk_toolbox.gtk_lock():
-			self.load_settings(config)
-
-		self.attempt_login(2)
 
 	def attempt_login(self, numOfAttempts = 10, force = False):
 		"""
@@ -342,6 +354,11 @@ class Dialcentral(object):
 			with gtk_toolbox.gtk_lock():
 				self._errorDisplay.push_exception(e)
 
+	def _spawn_attempt_login(self, *args):
+		backgroundLogin = threading.Thread(target=self.attempt_login, args=args)
+		backgroundLogin.setDaemon(True)
+		backgroundLogin.start()
+
 	def refresh_session(self):
 		"""
 		@note Thread agnostic
@@ -572,9 +589,7 @@ class Dialcentral(object):
 		if status == conic.STATUS_CONNECTED:
 			self._deviceIsOnline = True
 			if self._initDone:
-				backgroundLogin = threading.Thread(target=self.attempt_login, args=[2])
-				backgroundLogin.setDaemon(True)
-				backgroundLogin.start()
+				self._spawn_attempt_login(2)
 		elif status == conic.STATUS_DISCONNECTED:
 			self._deviceIsOnline = False
 			if self._initDone:
@@ -608,9 +623,7 @@ class Dialcentral(object):
 		self._contactsViews[self._selectedBackendId].clear()
 		self._change_loggedin_status(self.NULL_BACKEND)
 
-		backgroundLogin = threading.Thread(target=self.attempt_login, args=[2, True])
-		backgroundLogin.setDaemon(True)
-		backgroundLogin.start()
+		self._spawn_attempt_login(2, True)
 
 	def _on_notebook_switch_page(self, notebook, page, page_num):
 		if page_num == self.RECENT_TAB:
